Require Import List.
Require Import Program.
Set Implicit Arguments.

Program Definition 
  noSome(A:Type)(a:option A| a <> None) : { x:A | Some x = a } := 
  match a with 
    | None => _ 
    | Some x => x
  end.

Lemma someNoSomeId: 
  forall(A:Type)(a:option A)(H:a<>None),
    Some(proj1_sig(noSome(exist _ a H))) = a.
Proof.
  intros A a Ha;
    destruct a; simpl; auto; elim Ha; auto.
Qed.

Lemma someInjectiveEq: 
  forall(A:Type)(x y:A), 
    x = y <-> Some x = Some y.
Proof.
  intros A x y; split;intro H; [
    f_equal | inversion H ]; auto.
Qed.

Hint Rewrite someNoSomeId: option.

Fixpoint mapFilterSome (A B :Type)(f: A->option B)(l:list A) := 
  match l with
    | nil => nil
    | cons h t => 
      match (f h) with 
	| Some x => x::mapFilterSome f t
        | None => mapFilterSome f t
      end
  end.

Definition isSome (A:Type)(x:option A) : bool := 
  match x with 
    | None => false 
    | Some _ => true
  end.

Lemma isSomeTrueNotNone:
  forall (A:Type)(x: option A), 
    isSome x = true -> x<>None.
Proof.
  destruct x; intro H; discriminate.
Qed.

Lemma mapFilterSomeProp: 
  forall (A B :Type)(f: A->option B)(l:list A),
    map Some (mapFilterSome f l) = filter (@isSome B) (map f l).
Proof.
  induction l.
  auto.
  case_eq (f a);
    try(intros b Hfa);(try intros Hfa); simpl; rewrite Hfa; simpl; f_equal; assumption.
Qed.

