
Unset Automatic Introduction.    (* coq 8.3_rc1 *)
(** * Facts about length of lists *)

Require Import List.
Require Import Omega.

Open Scope list_scope.

Lemma lengthSig: 
  forall (A:Type)(P:list A->Prop)(l:list A)(Hl:P l),
    length(proj1_sig (exist P l Hl)) = length l.
Proof.
  intros A P l Hl.
  auto.
Qed.    

Hint Rewrite lengthSig: length.

(** if [l] has length [0], then [l=nil] *)

Lemma lengthNil : 
  forall (A:Type)(l : list A), 
    length l=0 -> 
    l=nil.
Proof.
  intros A0 l lenl.
  induction l.
  reflexivity.
  simpl in lenl. 
  absurd  (S (length l) = 0).
  omega. assumption.
Qed.

(** if [l] has length [0] iff [l=nil] *)
Lemma lengthNilEquiv : 
  forall (A:Type)(l : list A), 
    length l=0 <-> 
    l=nil.
Proof.
  intros A0 l; split ; 
    [ apply lengthNil | intro ; subst; reflexivity ].
Qed.

Global Ltac lengthNil :=
  match goal with 
    | [lenl : 0 = length ?l|-_]=>
      symmetry in lenl; rewrite (lengthNil l lenl) in *
    | [lenr : length ?l=0|-_]=>
      rewrite (lengthNil l lenr) in *
  end. 

(** if [l] has length > [0], then [l=a::l1] *)
Lemma lengthPos : 
  forall (A:Type)(l : list A) n , 
    length l=S n -> 
    exists a, exists l1, l=a::l1.
Proof.
  intros A0 l n lenl.
  induction l.
  simpl in lenl; discriminate.
  exists a. exists l. 
  reflexivity.
Qed.

(* TODO: Is it really usefull ? The next lemma seems sufficient. *)
Lemma lengthCons: 
  forall (A B:Set)(a1:A)(l1:list A)(l:list B),
  length(a1::l1)=length l -> 
  exists b:B,exists l2:list B,l=b::l2.
Proof.
  intros A B a1 l1 l H.
  induction l1; destruct l.
  simpl in H ; discriminate H.
  exists b. exists l. reflexivity.
  simpl in H ; discriminate H.
  exists b. exists l. reflexivity.
Qed.

(** If the length of a list is positive, this list has at least one element *)  
Lemma lengthS: forall (A:Set)(l:list A)(n:nat),
  S n =length l -> exists a:A,exists la:list A,l=a::la.
Proof.
  intros A l.
  induction l ; intros n H.
  discriminate H. 
  exists a. exists l. reflexivity.
Qed.

Ltac lengthPos :=
  match goal with 
    | [lenl : S ?n = length ?l|-_]=>
      symmetry in lenl; rewrite (lengthPos l lenl) in *
    | [lenl : length ?l= S ?n|-_]=>
      rewrite (lengthPos l lenl) in *
    | [lenl : length ?l > ?n|-_]=>
      rewrite (S_pred  (length l));
        rewrite (lengthPos l lenl) in *
  end. 

Implicit Arguments lengthNil [A].
Implicit Arguments lengthPos [A n].

Hint Rewrite app_length rev_length map_length fold_left_length : length.
Hint Rewrite  split_length_l split_length_r combine_length prod_length seq_length  : length.

Lemma lengthValueIndependent: forall (A:Type)(l1 l2:list A)(a b:A),
  List.length (l1++a::l2) = List.length(l1++b::l2).
Proof.
  intros A l1 l2 a b.
  autorewrite with length.
  trivial.
Qed.
